import { _decorator, Component, Node, game, input, Input, KeyCode, EventKeyboard, Vec3, SphereCollider, ICollisionEvent, RigidBody } from 'cc';
import { globalVars } from './globalVars';
const { ccclass, property } = _decorator;

@ccclass('PlayerScript')
export class PlayerScript extends Component {

    /* Local Variables */

        private movementSpeed = 3;
        private jumpStrength = 300;
        private crouch = 0;
        private moveForward = 0;
        private moveBackward = 0;
        private moveLeft = 0;
        private moveRight = 0;
        private setSpeed = 0.12;
        private maxSpeed = 0;
        private speed = 1;
        private grounded = false;
        private collider: SphereCollider = null!;
        private playerHeight = 2;

    /* End Local Variables */
    
    start() {
        globalVars.character = this;
        input.on(Input.EventType.KEY_DOWN, this.onKeyDown, this);
        input.on(Input.EventType.KEY_UP, this.onKeyUp, this)
        this.node.getComponent(SphereCollider).on('onCollisionStay', this.OnCollisionStay, this);
        this.node.getComponent(SphereCollider).on('onCollisionExit', this.OnCollisionExit, this);
        this.node.getComponent(SphereCollider).radius = this.playerHeight;
        this.maxSpeed = this.setSpeed;
    }

    OnCollisionStay(event: ICollisionEvent) {
        event.contacts.forEach(contact => {
            let out: Vec3 = new Vec3();
            contact.getWorldNormalOnB(out);
            if (Vec3.angle(out, Vec3.UP) < 60) {
                this.grounded = true;
            }
        });
    }

    OnCollisionExit() {
        this.grounded = false;
    }

    onKeyDown(event: EventKeyboard){
        switch(event.keyCode) {
            case KeyCode.KEY_A: {
                this.moveLeft = 1;
            break;}

            case KeyCode.KEY_W: {
                this.moveForward = 1;
            break;}

            case KeyCode.KEY_D: {
                this.moveRight = 1;
            break;}

            case KeyCode.KEY_S: {
                this.moveBackward = 1;
            break;}

            case KeyCode.SHIFT_LEFT: {
                this.speed = 1.1;
            break;}

            case KeyCode.KEY_C: {
                if(this.crouch == 0){
                    this.crouch = 1;
                    this.node.getComponent(RigidBody).clearForces();
                    this.node.getComponent(SphereCollider).radius = this.playerHeight/1.4;
                    this.maxSpeed = this.setSpeed/1.4;
                } else if(this.crouch == 1){
                    this.crouch = 0;
                    this.node.getComponent(SphereCollider).radius = this.playerHeight;
                    this.maxSpeed = this.setSpeed;
                }
            break;}

            case KeyCode.SPACE: {
                if(this.grounded && this.crouch == 0){
                    this.node.getComponent(RigidBody).applyLocalForce(new Vec3(0,this.jumpStrength*this.node.getComponent(RigidBody).mass,0), new Vec3(0,10,0));
                }
            break;}
        }
    }

    onKeyUp(event: EventKeyboard){
        switch(event.keyCode) {
            case KeyCode.KEY_A: {
                this.moveLeft = 0;
            break;}

            case KeyCode.KEY_W: {
                this.moveForward = 0;
            break;}

            case KeyCode.KEY_D: {
                this.moveRight = 0;
            break;}

            case KeyCode.KEY_S: {
                this.moveBackward = 0;
            break;}

            case KeyCode.SHIFT_LEFT: {
                this.speed = 1;
            break;}
        }
    }

    update(deltaTime: number) {
            if(this.moveForward && this.moveLeft){
                this.node.getComponent(RigidBody).applyLocalImpulse(new Vec3(-((this.maxSpeed/2)*this.node.getComponent(RigidBody).mass)*this.speed,0,-((this.maxSpeed/2)*this.node.getComponent(RigidBody).mass)*this.speed));
            } else if(this.moveForward && this.moveRight){
                this.node.getComponent(RigidBody).applyLocalImpulse(new Vec3(((this.maxSpeed/2)*this.node.getComponent(RigidBody).mass)*this.speed,0,-((this.maxSpeed/2)*this.node.getComponent(RigidBody).mass)*this.speed));
            } else if(this.moveBackward && this.moveLeft){
                this.node.getComponent(RigidBody).applyLocalImpulse(new Vec3(-((this.maxSpeed/2)*this.node.getComponent(RigidBody).mass)*this.speed,0,((this.maxSpeed/4)*this.node.getComponent(RigidBody).mass)*this.speed));
            } else if(this.moveBackward && this.moveRight){
                this.node.getComponent(RigidBody).applyLocalImpulse(new Vec3(((this.maxSpeed/2)*this.node.getComponent(RigidBody).mass)*this.speed,0,((this.maxSpeed/4)*this.node.getComponent(RigidBody).mass)*this.speed));
            } else if(this.moveForward){
                this.node.getComponent(RigidBody).applyLocalImpulse(new Vec3(0,0,-(this.maxSpeed*this.node.getComponent(RigidBody).mass)*this.speed));
            } else if(this.moveBackward){
                this.node.getComponent(RigidBody).applyLocalImpulse(new Vec3(0,0,((this.maxSpeed/2)*this.node.getComponent(RigidBody).mass)*this.speed));
            } else if(this.moveLeft){
                this.node.getComponent(RigidBody).applyLocalImpulse(new Vec3(-(this.maxSpeed*this.node.getComponent(RigidBody).mass)*this.speed,0,0));
            } else if(this.moveRight){
                this.node.getComponent(RigidBody).applyLocalImpulse(new Vec3((this.maxSpeed*this.node.getComponent(RigidBody).mass)*this.speed,0,0));
            }
    }
}