System.register(["cc"], function (_export, _context) {
  "use strict";

  var _cclegacy, _decorator, Component, Vec3, _dec, _class, _crd, ccclass, property, globalVars;

  function SmoothDamp(current, target, currentVelocity, smoothTime, maxSpeed = Infinity, deltaTime) {
    let output_x = 0;
    let output_y = 0;
    let output_z = 0; // Based on Game Programming Gems 4 Chapter 1.10

    deltaTime = Math.max(0.0001, deltaTime);
    smoothTime = Math.max(0.0001, smoothTime);
    let omega = 2 / smoothTime;
    let x = omega * deltaTime;
    let exp = 1 / (1 + x + 0.48 * x * x + 0.235 * x * x * x);
    let change_x = current.x - target.x;
    let change_y = current.y - target.y;
    let change_z = current.z - target.z;
    let originalTo = target.clone(); // Clamp maximum speed

    let maxChange = maxSpeed * smoothTime;
    let maxChangeSq = maxChange * maxChange;
    let sqrmag = change_x * change_x + change_y * change_y + change_z * change_z;

    if (sqrmag > maxChangeSq) {
      var mag = Math.sqrt(sqrmag);
      change_x = change_x / mag * maxChange;
      change_y = change_y / mag * maxChange;
      change_z = change_z / mag * maxChange;
    }

    target.x = current.x - change_x;
    target.y = current.y - change_y;
    target.z = current.z - change_z;
    let temp_x = (currentVelocity.x + omega * change_x) * deltaTime;
    let temp_y = (currentVelocity.y + omega * change_y) * deltaTime;
    let temp_z = (currentVelocity.z + omega * change_z) * deltaTime;
    currentVelocity.x = (currentVelocity.x - omega * temp_x) * exp;
    currentVelocity.y = (currentVelocity.y - omega * temp_y) * exp;
    currentVelocity.z = (currentVelocity.z - omega * temp_z) * exp;
    output_x = target.x + (change_x + temp_x) * exp;
    output_y = target.y + (change_y + temp_y) * exp;
    output_z = target.z + (change_z + temp_z) * exp; // Prevent overshooting

    let origMinusCurrent_x = originalTo.x - current.x;
    let origMinusCurrent_y = originalTo.y - current.y;
    let origMinusCurrent_z = originalTo.z - current.z;
    let outMinusOrig_x = output_x - originalTo.x;
    let outMinusOrig_y = output_y - originalTo.y;
    let outMinusOrig_z = output_z - originalTo.z;

    if (origMinusCurrent_x * outMinusOrig_x + origMinusCurrent_y * outMinusOrig_y + origMinusCurrent_z * outMinusOrig_z > 0) {
      output_x = originalTo.x;
      output_y = originalTo.y;
      output_z = originalTo.z;
      currentVelocity.x = (output_x - originalTo.x) / deltaTime;
      currentVelocity.y = (output_y - originalTo.y) / deltaTime;
      currentVelocity.z = (output_z - originalTo.z) / deltaTime;
    }

    return new Vec3(output_x, output_y, output_z);
  }

  _export("SmoothDamp", SmoothDamp);

  return {
    setters: [function (_cc) {
      _cclegacy = _cc.cclegacy;
      _decorator = _cc._decorator;
      Component = _cc.Component;
      Vec3 = _cc.Vec3;
    }],
    execute: function () {
      _crd = true;

      _cclegacy._RF.push({}, "ee02cU0FrlHAaK517mF6ygN", "globalVars", undefined);

      ({
        ccclass,
        property
      } = _decorator);

      _export("globalVars", globalVars = (_dec = ccclass('globalVars'), _dec(_class = class globalVars extends Component {
        constructor(...args) {
          super(...args);
          this.character = void 0;
          this.camera = void 0;
          this.start = 0;
        }
        /* End Global Variables */


      }) || _class));

      _cclegacy._RF.pop();

      _crd = false;
    }
  };
});
//# sourceMappingURL=bb6325b6a7a4da841647074b47a63e6a8cb7a2c6.js.map